@@ -9,8 +9,7 @@ namespace Tomboy
 	{
 		static readonly string[] DefaultEnabledPlugins = {
 			"BacklinksPlugin", "EvolutionPlugin", "ExportToHTMLPlugin",
-			"FixedWidthPlugin", "NoteOfTheDayPlugin", "PrintPlugin",
-			"StickyNoteImporter"
+			"FixedWidthPlugin", "PrintPlugin", "StickyNoteImporter"
 		};
 
 		public const string ENABLE_SPELLCHECKING = "/apps/tomboy/enable_spellchecking";
@@ -57,10 +56,12 @@ namespace Tomboy
 		{
 			switch (key) {
 			case ENABLE_SPELLCHECKING:
-			case ENABLE_CUSTOM_FONT:
 			case ENABLE_KEYBINDINGS:
 				return true;
 
+			case ENABLE_CUSTOM_FONT:
+				return false;
+
 			case ENABLE_WIKIWORDS:
 				return false;
 			
@@ -637,6 +638,7 @@ namespace Tomboy
 				GuiUtils.MakeImageButton (Gtk.Stock.Preferences,
 							  Catalog.GetString ("Settings"));
 			plugin_preferences.Clicked += ShowPluginSettings;
+			plugin_preferences.Sensitive = false;
 
 			open_plugin_folder = 
 				GuiUtils.MakeImageButton (
@@ -668,6 +670,9 @@ namespace Tomboy
 
 		void ShowPluginSettings (object sender, EventArgs e)
 		{
+			if (current_plugin == null)
+				return;
+			
 			Gtk.Image icon = 
 				new Gtk.Image (Gtk.Stock.Preferences, Gtk.IconSize.Dialog);
 
